proc setPatColor { n } {
	global sld bk
	set sld(patcolor$n) [tk_chooseColor]
	.bk.1.b$n config -bg $sld(patcolor$n)
}

proc getBackground {} {
	global sld bk
	toplevel .bk
	wm title .bk "Background Pattern"
	wm transient .bk .
	pack [frame .bk.1] [frame .bk.2] [frame .bk.3] -side left
	pack \
	[label .bk.1.col1 -text "Begin color"] \
	[button .bk.1.b1 -width 1 -height 1 -font {Times 8} -bg $sld(patcolor1) \
		-command {setPatColor 1} ] \
	[label .bk.1.col2 -text "End color"] \
	[button .bk.1.b2 -width 1 -height 1 -font {Times 8} -bg $sld(patcolor2) \
			-command {setPatColor 2} ]\
	[button .bk.1.done -text OK -command {
		if {$bk(kind) == -1} {
			set sld(backpattern) ""
		} else {
			set sld(backpattern) [list $sld(patcolor1) $sld(patcolor2) $bk(kind)]
		}
		set sld(page,update) 1
		setBackground;destroy .bk}] \
	[button .bk.1.cancel -text Cancel -command { destroy .bk }] \
		-side top -anchor c -fill y


	pack \
	[radiobutton .bk.2.solid -bitmap @$sld(idir)/patnone.xbm \
		-variable bk(kind) -value -1] \
	[radiobutton .bk.2.0 -bitmap @$sld(idir)/pat0.xbm \
		-variable bk(kind) -value 0] \
	[radiobutton .bk.2.1 -bitmap @$sld(idir)/pat1.xbm \
		-variable bk(kind) -value 1] \
	[radiobutton .bk.2.2 -bitmap @$sld(idir)/pat2.xbm \
		-variable bk(kind) -value 2] -side top
	pack \
	[radiobutton .bk.3.3 -bitmap @$sld(idir)/pat3.xbm \
		-variable bk(kind) -value 3] \
	[radiobutton .bk.3.4 -bitmap @$sld(idir)/pat4.xbm \
		-variable bk(kind) -value 4] \
	[radiobutton .bk.3.5 -bitmap @$sld(idir)/pat5.xbm \
		-variable bk(kind) -value 5] \
	[radiobutton .bk.3.6 -bitmap @$sld(idir)/pat6.xbm \
		-variable bk(kind) -value 6] -side top

	centerWindow .bk
	raise .bk
}


proc setBackground {} {
	global sld
	if {$sld(backpattern) != ""} {
        .c.c delete -tags background 
        catch {image delete ibackground}
        image create photo ibackground \
            -width $sld(dwidth) -height $sld(dheight) -palette 16/16/16
		eval pattern ibackground $sld(backpattern)
		.c.c create image 0 0 -image ibackground \
			-anchor nw -tags background
		.c.c lower background
	} else {
		.c.c delete -tags background
		.c.c configure -background $sld(background)
		set sld(page,update) 1
	} 
}

proc setforeground {} {
	global sld
	set scolor [tk_chooseColor]
	if [string length $scolor] {
		set sld(foreground) $scolor
	}
}

proc setfillcolor {} {
	global sld
	set scolor [tk_chooseColor]
	if [string length $scolor] {
		set sld(fillcolor) $scolor
	}
}

proc deleteColor {pos clr} {
	global sld
	destroy .pal.c.$pos
	set el [lsearch $sld(colorlist) $clr]
	set list1 [lrange $sld(colorlist) 0 [expr $el - 1]]
	set list2 [lrange $sld(colorlist) [expr $el + 1] end]	
	set sld(colorlist) [concat $list1 $list2]
}

proc setColor {clr} {
	global sld
	if "[string length $clr] == 0" { 
		return 
	}
	set sld($sld(where)) $clr
	.c.c configure -background $sld(background)
	if [expr [lsearch $sld(colorlist) $clr] >= 0] {
	} {
		lappend sld(colorlist) $clr
		makeColorButtons
	}
}

#
# makeColorButtons -- creates one button for each color in the palette
#
proc makeColorButtons {} {
	global sld
	catch {destroy .pal.c}
	frame .pal.c 
	set pos 0
	foreach clr $sld(colorlist) {
		button .pal.c.$pos -font {Times 8} -bg $clr -width 1 -height 1 \
			-command "setColor $clr"  
		grid .pal.c.$pos -column [expr $pos%10] -row [expr $pos/10]
		bind .pal.c.$pos <2> "deleteColor $pos $clr"
		incr pos
	}
	pack .pal.c -side top
}

#
# showPalette -- makes a new toplevel for color selection or editing
# Each color added, will be appended to sld(colorlist) so it will be
# easy to reselect the same color again, when needed.
# sld(where) tell us which color (background, foreground or fillcolor) to
# change.  Colors may be choosen by name (see rgb.txt in your X11
# installation, for a list of color names), or by a standard RGB dialog.
#
proc showPalette {} {
	global sld
	toplevel .pal
	wm transient .pal .
	makeColorButtons
	set sld(where) foreground
	frame .pal.f
	radiobutton .pal.f.fore	-text Foreground \
		-variable sld(where) -value foreground
	radiobutton .pal.f.back	-text Background \
		-variable sld(where) -value background
	radiobutton .pal.f.fill	-text Fillcolor \
		-variable sld(where) -value fillcolor
	button .pal.f.cancel -text Dismiss -command {destroy .pal}
	frame .pal.b
	label .pal.b.l -text "By name"
	entry .pal.b.e
	button .pal.b.browse -text "Browse..." -command {setColor [tk_chooseColor]}
	pack .pal.b.l -side left
	pack .pal.b.e -expand 1 -fill x -side left
	pack .pal.b.browse -side left
	pack .pal.f.fore .pal.f.back .pal.f.fill .pal.f.cancel -side left
	pack .pal.b .pal.f -side bottom
	.pal.f.fore invoke
	bind .pal.b.e <Return> {setColor [.pal.b.e get]}
	centerWindow .pal
	raise .pal
}

