#
# Slide files are simply tar files with slides numbered 1,2,3...
#
# variables:
# ----------
# sld(page,current) - current slide (integer)
# sld(page,max) - last slide in the tarball
# sld(page,update) - flags if the slide need to be flushed
#   *** procs that turn on update flag: objectStartMove, objectOptions,
#	cutObject, pasteObject, frontObject, backObject, groupObjects, 
#	ungroupObjects, insertLine, insertTextMark, insertImage.
#	(be aware of this when introducing another editing procedure)
# The directory $sld(dir)/tmp is used for temporary storage
# of slide pages.
#

#
# fileNew
#
proc fileNew {} {
	global sld sldi cv
### give a chance to the user save his file
	if {($sld(update)||$sld(page,update)) && \
		([tk_dialog .svquit "Leaving Slidedraw" \
			"Do you want to save current slide file before starting a new one?" \
			questhead 0 Save Quit] == 0)} {
				fileSave
	} 
	.c.c delete slide
	set sld(page,update) 0
	set sld(page,max) 1
	set sld(page,current) 1
	set sld(update) 0
	cleanSlideDir
	statusSlide
}

#
# fileSave -- try to save the file, guessing its name or asking the user
#
proc fileSave {} {
	global sld sldi cv
	updateSlide $sld(page,current)
# if no filename found, try "noname"
	if {![info exists sld(currfile)]} {
		set sld(currfile) $sld(dir)/slides/noname.sld
	} 
# ask the user for its name
	regexp "(.*/)(.*)" $sld(currfile) fullname path fname
	set fn [tk_getSaveFile -initialdir $path -initialfile $fname \
	-defaultextension ".sld" -filetypes $sld(filetypes)]
	set sld(currfile) $fn
# add all slide files to the tarball
	set olddir [exec pwd]
	cd $sld(dir)/tmp
	eval exec tar cvzf $fn [glob *]
	cd $olddir
# updates file list menu
	fileListUpdate $fn
}

#
# fileMerge -- Merge loaded file with the present drawing, so it becomes 
# easier to make several similar slides :) Deprecated.
#
# proc fileMerge {} {
# 	global sld sldi cv
#	source [tk_getOpenFile]	
#	.c.c config -bg $sld(background)
# }

#
# fileLoad -- clean slide, get a filename and "source" it.
# Then update the list of files recently loaded. 
#
proc fileLoad { { fn "" } } {
	global sld sldi cv
# if no filename found, try "noname"
	if {![info exists sld(currfile)]} {
		set sld(currfile) $sld(dir)/slides/noname
	} 
	regexp "(.*/)(.*)" $sld(currfile) fullname path fname
	if {[string length $fn] == 0} { set fn [tk_getOpenFile -initialdir $path \
	-initialfile $fname -defaultextension ".sld" -filetypes $sld(filetypes)] }
### if still no name, user has canceled the loading
	if {[string length $fn] == 0} { return }
	fileNew
### expand tarball to tmp directory
	set olddir [exec pwd]
	cd $sld(dir)/tmp
	eval exec tar xvzf $fn
	cd $olddir
	set sld(currfile) $fn
	fileListUpdate $fn
### update sld(page,max)
	set sld(page,max) [llength [glob $sld(dir)/tmp/\[1-9\]*]]
	set sld(page,current) 1
	showSlide
}

#
# show one slide or all slides if in multislide view
#
proc showSlide { } {
	global sld cv
	if {$sld(multi)} {
		set n 1
		for {set i 0} {$i < $sld(ymulti)} {incr i} {
			for {set j 0} {$j < $sld(xmulti)} {incr j} {
				if {$n > $sld(page,max)} {
					break
				}
				set cv .c.x${i}y$j
				showOneSlide $n
				incr n
				update idletasks
			}
		}
	} else {
		set cv .c.c
		showOneSlide $sld(page,current)
	}
}

proc cleanSlideDir { } {
	global sld sldi cv
	catch {
		foreach f [glob $sld(dir)/tmp/*] {
			exec rm $f
		}
	}
}

proc updateSlide { n } {
	global sld sldi cv
	if {$sld(page,update)} {
		set sld(update) 1
		set fd [open $sld(dir)/tmp/$n w]
# get each object description and write to the slide file
		foreach i [.c.c find withtag slide] {
			puts $fd [describeObj $i]
		}
# save also all variables in sld(svarlist), that describes some properties
# of this particular slide
		foreach i $sld(svarlist) {
			puts $fd "set sld($i) \"$sld($i)\""
		}
		close $fd
		set sld(page,update) 0
	}
# erase all selected objects (for grouping)
	$cv delete withtag flaggrp
	set sld(group) {}
}

proc showOneSlide { n } {
	global sld sldi sldimg sldfnt cv twidth
	$cv delete slide
	catch {unset sldi}
	catch {unset sldimg}
	catch {unset sldfnt}
	set twidth [expr $sld(textwidth)*$sld(scale)]
	if {[file exists $sld(dir)/tmp/$n]} {
		source $sld(dir)/tmp/$n
		$cv config -bg $sld(background)
		if {$sld(multi)} {
			$cv scale slide 30 30 $sld(scale) $sld(scale)
			foreach i [$cv find withtag slide] {
				switch 	[$cv type $i] {
				"text" {
					set fnt [$cv itemcget $i -font]
					set fsiz [expr round([lindex $fnt 1] * $sld(scale))]
					set fnt [lreplace $fnt 1 1 $fsiz] 
					$cv itemconfigure $i -font $fnt
				}
				"image" {
					set img [$cv itemcget $i -image]
					image create photo z$img
					if {$sld(scale) < 1.0} {
						z$img copy $img -subsample [expr round(1/$sld(scale))]
					} else {
						set zscale [expr round($sld(scale))]
						z$img copy $img -zoom $zscale $zscale					
					}
					$cv itemconfigure $i -image z$img
				}	
				}		
			}
		} else {
			$cv scale slide 0 0 $sld(scale) $sld(scale)
			foreach i [$cv find withtag slide] {
				switch 	[$cv type $i] {
				"text" {
					if [array exists sldfnt($i)] {
						set fnt $sldfnt($i)
					} else {
						set fnt [$cv itemcget $i -font]
					}
					set fsiz [expr round([lindex $fnt 1] * $sld(scale))]
					set showfnt [lreplace $fnt 1 1 $fsiz] 
					$cv itemconfigure $i -font $showfnt
					set sldfnt($i) $fnt
				}
				"image" {
					if [info exists sldimg($i)] {
						set img $sldimg($i)
					} else {
						set img [$cv itemcget $i -image]
					}
					catch { image delete z$img }
					image create photo z$img
					if {$sld(scale) < 1.0} {
						z$img copy $img -subsample [expr round(1/$sld(scale))]
					} else {
						set zscale [expr round($sld(scale))]
						z$img copy $img -zoom $zscale $zscale						
					}
					$cv itemconfigure $i -image z$img
					set sldimg($i) $img
				}	
				}		
			}
            setupCanvasSize
		}
	}
	statusSlide
	setBackground
}

#
# nextSlide, prevSlide -- navigation events for control with the mouse
#
proc gotoSlide { n } {
	global sld sldi cv
	if {$n <= $sld(page,max)} {
		updateSlide $sld(page,current)
		set sld(page,current) $n
		showOneSlide $sld(page,current)
	}
}

proc nextSlide { } {
	global sld sldi cv
	unhiliteItem
	if {!$sld(page,update) && ![llength [$cv find withtag slide]] &&
		($sld(page,current)==$sld(page,max))} {
			return
	}
	updateSlide $sld(page,current)
	incr sld(page,current)	
	if {$sld(page,current) > $sld(page,max)} {
		incr sld(page,max)
	}
	showOneSlide $sld(page,current)
}

proc prevSlide { } {
	global sld sldi cv
	unhiliteItem
	if {$sld(page,current) == 1} { return }
### if we're at the last page and it's clean, delete it
	if {($sld(page,current) == $sld(page,max)) &&
		![llength [$cv find withtag slide]]} {
			incr sld(page,max) -1
	} else {
		updateSlide $sld(page,current)
	}
	incr sld(page,current) -1
	showOneSlide $sld(page,current)
}

proc duplSlide {} {
	global sld sldi cv
	updateSlide $sld(page,current)
	copyAllObjects
	set n [expr $sld(page,max)+1]
	set sld(page,current) $n
	if {$sld(page,current) > $sld(page,max)} {
		incr sld(page,max)
	}
	showOneSlide $sld(page,current)
	pasteAllObjects
}

proc newSlide {} {
	global sld sldi cv
	updateSlide $sld(page,current)
	set tdir $sld(dir)/tmp
	for {set i $sld(page,max)} {$i >= $sld(page,current)} {incr i -1} {
		file rename $tdir/$i $tdir/[expr $i+1]
	}
	incr sld(page,max)
	showOneSlide $sld(page,current)
	
}

proc removeSlide {} {
	global sld sldi cv
	set sld(page,update) 0
	set sld(update) 1
	set tdir $sld(dir)/tmp
	catch {file delete $tdir/$sld(page,current)}
	if {$sld(page,current) != $sld(page,max)} {
		for {set i [expr $sld(page,current)+1]} \
			{$i <= $sld(page,max)} {incr i} {
				file rename $tdir/$i $tdir/[expr $i-1] 
		}
		incr sld(page,max) -1
		gotoSlide $sld(page,current)
	} else {
		foreach i [$cv find withtag slide] {
			$cv delete $i
		}
		prevSlide
	}
}

proc statusSlide { } {
	global sld sldi cv
	set s [format "slide: $sld(page,current)/$sld(page,max)"]
	.cnt config -text $s
}
