#
# makeGUI -- creates all main GUI widgets
# and loads images for the button bar
#
proc makeGUI { } {
global sld cv
frame .mb
frame .bar
frame .c
## width and height are the sizes of the _widget_, not the image edited
canvas .c.c -bg $sld(background) -width $sld(width) -height $sld(height) \
	-scrollregion [list -1c -1c $sld(dwidth) $sld(dheight)] \
	-confine 1 -closeenough 15.0 \
	-xscrollcommand {.c.cscrollh set} -yscrollcommand {.c.cscrollv set} \
	-xscrollincrement 1 -yscrollincrement 1

scrollbar .c.cscrollh -orient horizontal -width 10 -command {.c.c xview}
scrollbar .c.cscrollv -orient vertical -width 10 -command {.c.c yview}
label .cnt -relief groove -width 12
label .st -relief sunken

set cv .c.c

wm geometry . +5+0 

menubutton .mb.file -text File -underline 0 -menu .mb.file.m
menubutton .mb.edit -text Edit -underline 0 -menu .mb.edit.m
menubutton .mb.layout -text Layout -underline 0 -menu .mb.layout.m
menubutton .mb.objects -text Objects -underline 0 -menu .mb.objects.m
menubutton .mb.view -text View -underline 0 -menu .mb.view.m
#menubutton .mb.effects -text Effects -underline 5 -menu .mb.effects.m \
#		-state disabled
menubutton .mb.help -text Help -underline 0 -menu .mb.help.m


############################ FILE

menu .mb.file.m 
.mb.file.m add command -label "New" -accelerator Ctl+N -underline 0 \
	-command fileNew
.mb.file.m add command -label "Open..." -accelerator Clt+O -underline 0 \
	-command fileLoad
.mb.file.m add separator
.mb.file.m add command -label "Save as..." -accelerator Ctl+S -underline 0 \
	-command fileSave
.mb.file.m add command -label "Export..." -underline 0 \
	-command fileExport
.mb.file.m add command -label "Page Setup..." -underline 0 -command page
.mb.file.m add command -label "Print..." -underline 0 \
	-command getPrintOptions
.mb.file.m add command -label "Exit" -accelerator Ctl+Q -underline 0 \
	-command "saveConfig; exit"
.mb.file.m add separator


################################## EDIT

menu .mb.edit.m
.mb.edit.m add command -label "Cut" -accelerator "     Ctl+X" -underline 2 \
	-command cutObject
.mb.edit.m add command -label "Copy" -accelerator "     Ctl+C" -underline 0 \
	-command copyObject
.mb.edit.m add command -label "Paste" -accelerator "     Ctl+V" -underline 0 \
	-command pasteObject
.mb.edit.m add command -label "Copy All" -underline 5 \
	-command copyAllObjects
.mb.edit.m add separator
.mb.edit.m add command -label "Bring to front" -accelerator "     Ctl+F" -underline 9 \
	-command frontObject
.mb.edit.m add command -label "Send to back" -accelerator "     Ctl+B" -underline 9 \
	-command backObject
.mb.edit.m add command -label "Group objects" -accelerator "     Ctl+G" -underline 0 \
	-command groupObjects
.mb.edit.m add command -label "Ungroup objects" -accelerator "Shft+Ctl+G" \
	-command ungroupObjects
	
################################# LAYOUT

menu .mb.layout.m
### Text layout
menu .mb.layout.m.font
.mb.layout.m add cascade -label "Text layout" -menu .mb.layout.m.font
.mb.layout.m.font add command -label "Font browse..." -command setFontDialog
.mb.layout.m.font add cascade -label "Font family" -menu .mb.layout.m.font.ff
.mb.layout.m.font add cascade -label "Font size" -menu .mb.layout.m.font.fs
.mb.layout.m.font add cascade -label "Text alignment" -menu .mb.layout.m.font.ta

### Page layout

### Document layout...
#
# Mmmm... Do we have Document layout as an entity per se?
# Here we should have general properties for the whole presentation...
.mb.layout.m add cascade -label "Grid resolution" -menu .mb.layout.m.s

### Color layout
.mb.layout.m add command -label "Color palette..." -underline 0 \
	-command showPalette
.mb.layout.m add command -label "Background pattern..." -command getBackground
# .mb.layout.m add separator

### Object layout
menu .mb.layout.m.obj
.mb.layout.m add cascade -label "Object layout" -menu .mb.layout.m.obj  
.mb.layout.m.obj add cascade -label "Line width" -menu .mb.layout.m.obj.w
.mb.layout.m.obj add cascade -label "Line style" -menu .mb.layout.m.obj.sty


################# TEXT LAYOUT
menu .mb.layout.m.font.ff
set index 0
foreach family $sld(fontfamilies) {
	.mb.layout.m.font.ff add radiobutton -label $family -underline 0 \
		-variable sld(fontfamily) -value $family
	incr index
}

menu .mb.layout.m.font.fs
.mb.layout.m.font.fs add radiobutton -label 12 -underline 1 \
	-variable sld(fontsize) -value 12
.mb.layout.m.font.fs add radiobutton -label 14 -underline 1 \
	-variable sld(fontsize) -value 14
.mb.layout.m.font.fs add radiobutton -label 18 -underline 1 \
	-variable sld(fontsize) -value 18
.mb.layout.m.font.fs add radiobutton -label 24 \
	-variable sld(fontsize) -value 24
.mb.layout.m.font.fs add radiobutton -label 36 \
	-variable sld(fontsize) -value 36
.mb.layout.m.font.fs add radiobutton -label 48 \
	-variable sld(fontsize) -value 48
.mb.layout.m.font.fs add radiobutton -label 72 \
	-variable sld(fontsize) -value 72

menu .mb.layout.m.font.ta
.mb.layout.m.font.ta add radiobutton -label "Left text" \
	-variable sld(txtalign) -value left
.mb.layout.m.font.ta add radiobutton -label "Right align"\
	-variable sld(txtalign) -value right
.mb.layout.m.font.ta add radiobutton -label "Center align" \
	-variable sld(txtalign) -value center
	
#### set default fontsize and fontfamily (first)
.mb.layout.m.font.ff invoke 1
.mb.layout.m.font.fs invoke 5
.mb.layout.m.font.ta invoke 1

############## PAGE is defined in utils.tcl

############## DOCUMENT LAYOUT

menu .mb.layout.m.s
.mb.layout.m.s add radiobutton -label "Size 1" \
	-variable sld(snap) -value 1
.mb.layout.m.s add radiobutton -label "Size 2" \
	-variable sld(snap) -value 2
.mb.layout.m.s add radiobutton -label "Size 4" \
	-variable sld(snap) -value 4
.mb.layout.m.s add radiobutton -label "Size 8" \
	-variable sld(snap) -value 8
.mb.layout.m.s add radiobutton -label "Size 16" \
	-variable sld(snap) -value 16
.mb.layout.m.s add radiobutton -label "Size 32" \
	-variable sld(snap) -value 32	

.mb.layout.m.s invoke 4

############## OBJECT LAYOUT
menu .mb.layout.m.obj.w
.mb.layout.m.obj.w add radiobutton -label "Width 1" \
	-variable sld(lwidth) -value 1
.mb.layout.m.obj.w add radiobutton -label "Width 2" \
	-variable sld(lwidth) -value 2
.mb.layout.m.obj.w add radiobutton -label "Width 3" \
	-variable sld(lwidth) -value 3
.mb.layout.m.obj.w add radiobutton -label "Width 4" \
	-variable sld(lwidth) -value 4
.mb.layout.m.obj.w add radiobutton -label "Width 6" \
	-variable sld(lwidth) -value 6
.mb.layout.m.obj.w add radiobutton -label "Width 8" \
	-variable sld(lwidth) -value 8
.mb.layout.m.obj.w add radiobutton -label "Width 12" \
	-variable sld(lwidth) -value 12

menu .mb.layout.m.obj.sty
.mb.layout.m.obj.sty add radiobutton -label "_____" \
	-variable sld(lstyle) -value " "
.mb.layout.m.obj.sty add radiobutton -label "....." \
	-variable sld(lstyle) -value "."
.mb.layout.m.obj.sty add radiobutton -label ".-.-." \
	-variable sld(lstyle) -value ".-"
.mb.layout.m.obj.sty add radiobutton -label ".. .. .." \
	-variable sld(lstyle) -value "..  "
.mb.layout.m.obj.sty add radiobutton -label "..-..-" \
	-variable sld(lstyle) -value "..-"
 
.mb.layout.m.obj.w invoke 4

#.mb.layout.m add separator
#.mb.layout.m add command -label footer -underline 0 -state disabled
#.mb.layout.m add command -label background -underline 0 -state disabled

##################################### OBJECTS
 
menu .mb.objects.m
.mb.objects.m add command -label Line -underline 0 \
	-command {doBindings line}
.mb.objects.m add command -label Frame -underline 0 \
	-command {doBindings frame}
.mb.objects.m add command -label Box -underline 0 \
	-command {doBindings box}
.mb.objects.m add command -label Ellipse -underline 0 \
	-command {doBindings ellipse}
.mb.objects.m add command -label "Filled ellipse" -underline 0 \
	-command {doBindings fellipse}
.mb.objects.m add command -label Text -underline 0 \
	-command {doBindings text}
.mb.objects.m add command -label "Boxed text" -underline 2 \
	-command {doBindings btext}
.mb.objects.m add command -label "Image" -underline 0 \
	-command {doBindings image}

	
#################################### VIEW
menu .mb.view.m
.mb.view.m add checkbutton -label "Show rulers" -variable sld(rulersenable) \
	-command {makeRulers}
.mb.view.m add checkbutton -label "Contact sheet" -underline 0\
	-variable sld(multi) -command showMulti
.mb.view.m add separator
.mb.view.m add cascade -label "Zoom" -underline 0 -menu .mb.view.m.z
.mb.view.m add separator
.mb.view.m add command -label "Next slide" -accelerator "PgDown" \
	-command nextSlide
.mb.view.m add command -label "Previous slide" -accelerator "PgUp" \
	-command prevSlide
.mb.view.m add cascade -label "Goto slide" -menu .mb.view.m.goto 
.mb.view.m add command -label "Duplicate slide" \
	-command duplSlide
.mb.view.m add command -label "Remove slide" \
	-command removeSlide
.mb.view.m add command -label "Insert slide" \
	-command newSlide
.mb.view.m add separator
.mb.view.m add command -label "Slideshow" -accelerator "Ctrl+F7"\
	-command slideshow

########### auxiliary menu GotoSlide
menu .mb.view.m.goto -tearoff 0 -postcommand menuGotoSlide

########### ZOOM
menu .mb.view.m.z
.mb.view.m.z add radiobutton -label "1:5" \
	-variable sld(zoom) -value 0.2 -command applyZoom
.mb.view.m.z add radiobutton -label "1:3" \
	-variable sld(zoom) -value 0.333 -command applyZoom
.mb.view.m.z add radiobutton -label "1:2.5" \
	-variable sld(zoom) -value 0.4 -command applyZoom
.mb.view.m.z add radiobutton -label "1:2" \
	-variable sld(zoom) -value 0.5 -command applyZoom
.mb.view.m.z add radiobutton -label "1:1" \
	-variable sld(zoom) -value 1.0 -command applyZoom
.mb.view.m.z add radiobutton -label "2:1" \
	-variable sld(zoom) -value 2.0 -command applyZoom
.mb.view.m.z add radiobutton -label "3:1" \
	-variable sld(zoom) -value 3.0 -command applyZoom
.mb.view.m.z add radiobutton -label "5:1" \
	-variable sld(zoom) -value 5.0 -command applyZoom
.mb.view.m.z add radiobutton -label "7:1" \
	-variable sld(zoom) -value 7.0 -command applyZoom
.mb.view.m.z add radiobutton -label "10:1" \
	-variable sld(zoom) -value 10.0 -command applyZoom
.mb.view.m.z add radiobutton -label "15:1" \
	-variable sld(zoom) -value 15.0 -command applyZoom




############################### HELP

menu .mb.help.m -tearoff 0
.mb.help.m add command -label "About..." -underline 0 \
	-command {tk_dialog .dlg "About Slidedraw"\
"(C)1998 Rildo Pragana. \
 (C)1998 Slidedraw Team.\n\
Distributed under GNU GPL. Please read COPYING. \
Suggestions, inquiries or comments\
 will be appreciated: <rpragana@acm.org>.\
 Please read the file README\
 for more information." "" 0 OK}


image create bitmap flaggrp -background yellow -foreground black \
	-file $sld(idir)/flaggrp.xbm -maskfile $sld(idir)/flaggrp-mask.xbm

#image create photo ibackground -width [winfo pixels .c.c $sld(dwidth)] \
#     -height [winfo pixels .c.c $sld(dheight)] -palette 6/6/6

################################# TOOL BAR

set idx 0
foreach btn $sld(toolbar) {
        if {[lindex $btn 0] == ""} {
                pack [frame .bar.s$idx -width 10] -side left
        } else {
                image create photo ib$idx -file $sld(idir)/[lindex $btn 0]
                pack [button .bar.b$idx -image ib$idx -command \
                        "[lindex $btn 1] invoke [lindex $btn 2]"] -side left
        }
        incr idx
}

########################## GEOMETRY MANAGER

### Menu bar
pack .mb.file .mb.edit .mb.layout .mb.objects .mb.view .mb.help -side left

pack .c.cscrollh -side bottom -fill x
pack .c.c -side left -fill both -expand 1 
pack .c.cscrollv -side right -fill y

pack .mb .bar -side top -expand 1 -fill x 
pack .c -side top -expand 1 -fill both
pack .cnt -side left 
pack .st -side bottom -expand 1 -fill x

## move position so to show rulers at the beginning
.c.c xview scroll 100 units
.c.c yview scroll 100 units

dashPatchCheck
}
