proc objectResize {x y} {
	global Resizing
	snap x y
	set Resizing 0
	if {[string match {[0-9 ]*} [objTag $x $y]]} {
		set sld(obj) [objTag $x $y]
		set xy [.c.c coords $sld(obj)]
		set dx [expr [lindex $xy 2] - 10]
		set dy [expr [lindex $xy 3] - 10]
		if {([llength $xy] == 4) && ($y > $dy) && ($x > $dx)} {
			.c.c config -cursor sizing
			set Resizing 1
		} {
			.c.c config -cursor left_ptr
		}
	}
}

proc objectStartMove {x y} {
	global sld
	set sld(page,update) 1
	snap x y
	set sld(group) {}
	.c.c delete withtag flaggrp
## save position where the mouse was clicked to calculate motion after
	set sld(x0) $x
	set sld(y0) $y
	set sld(obj) [objTag $x $y]
	.c.c bind $sld(obj) <B2-Motion> {objectMove %x %y} 
}

proc objectSelectAdd {x y} {
	global sld
	snap x y
	set sld(obj) {}
	set e [objTag $x $y]
	if {[lsearch $sld(group) $e] < 0} {
		lappend sld(group) $e
## put a flag to show which objects are part of the group
		foreach i [.c.c find withtag $e] {
			set pos [.c.c coords $i]
			.c.c create image [lindex $pos 0] [lindex $pos 1] \
				-image flaggrp -tags flaggrp
		}
	}
}

proc groupObjects {} {
	global sld
	set sld(page,update) 1
	set sld(obj) {}
	set objs {}
	foreach i $sld(group) {
		eval lappend objs [.c.c find withtag $i]
		.c.c dtag $i
	}
	foreach i $objs {
		.c.c addtag g$sld(groupcnt) withtag $i
	}
	.c.c delete withtag flaggrp
	incr sld(groupcnt)
}

proc ungroupObjects {} {
	global sld
	set sld(page,update) 1
	.c.c dtag $sld(group)
}

proc objTag {x y} {
	global sld
	set obj [.c.c find closest $x $y]
	foreach t [.c.c gettags $obj] {
# verifica se existe um tag de grupo "gxxx"
		if {[string index $t 0] == "g"} {
			return $t
		} 
	}
	return $obj
}

proc objectMove {x y} {
	global sld Resizing
	snap x y
## sld(obj) is a tag (grouped objects) or an index (single object)
	set xy [.c.c coords $sld(obj)]
## if not resizing, or object not resizable (have only x y coords)
	if {($Resizing == 0) || ([llength $xy] != 4)} {
		.c.c move $sld(obj) [expr $x - $sld(x0)] [expr $y - $sld(y0)]
## save new position to posterior motions
		set sld(x0) $x
		set sld(y0) $y
	} else {
## object is being resized, set new size (a rectangle with x1 y1 x2 y2)
		.c.c coords $sld(obj) [lindex $xy 0] [lindex $xy 1] $x $y
	}
}

#----------------- Object insertion operations ----------------
#
# insertImage -- creates a tk photo image with the file given and
# insert it in the canvas. Tk directly supports ppm/pgm or gif formats.
#
proc insertImage {x y} {
	global sld sldimg cv
	set sld(page,update) 1
	snap x y
## now this is done in separate image dialog
	set imgname [makeImage]
	if {$imgname != ""} {
		catch { image delete z$imgname }
		image create photo z$imgname
		if {$sld(scale) < 1.0} {
			z$imgname copy $imgname -subsample [expr round(1/$sld(scale))]
		} else {
			set zscale [expr round($sld(scale))]
			z$imgname copy $imgname -zoom $zscale $zscale						
		}
		set sld(obj) [.c.c create image $x $y -anchor c \
			-image z$imgname -tags slide]
		set sldimg($sld(obj)) $imgname
	}
}

#
# insertTextDone -- terminates text (or boxed text) insertion
# destroying the dialog
#
proc insertTextdone {x y} {
	global sld sldfnt boxedtext
#	snap x y     <-- wrong (don't snap again!)
	set t [.txt.e get]
	set fontdesc [list $sld(fontfamily) $sld(fontsize) $sld(fontstyle)]
	set showfnt [list $sld(fontfamily) \
		[expr round($sld(fontsize)*$sld(scale))] $sld(fontstyle)]
	if {$boxedtext} {
		set sld(obj) [.c.c create text $x $y -anchor nw -text $t \
			-font $showfnt \
			-fill $sld(foreground) -width $sld(textwidth) \
			-justify $sld(txtalign) -tags slide]
	} {
		set sld(obj) [.c.c create text $x $y -anchor sw -text $t \
			-font $showfnt \
			-fill $sld(foreground) -justify $sld(txtalign) -tags slide]
	}
	set sldfnt($sld(obj)) $fontdesc
	destroy .txt
}

#
# insertText -- creates a simple dialog (label: entry)
# and dispatches insertTextDone when finished
#
proc insertText { x y } {
	global sld boxedtext
	if {$boxedtext == 0} {
		snap x y
	}
	toplevel .txt -bd 3 -relief ridge
	wm transient .txt .
	label .txt.l -text "Text:"
	entry .txt.e -width 40
	pack .txt.l .txt.e -side left
	centerWindow .txt
	raise .txt
	grab -global .txt
	focus .txt.e
	bind .txt.e <Return> "insertTextdone $x $y"
	bind .txt <Leave> {destroy .txt}
}

#
# insertTextMark, insertTextDrag, 
# insertTextLimit -- process events for text insertion
#
# insertTextMark  --> mark beginning of insertion (1st point)
#                     when button-1 is clicked
# insertTextDrag  --> changes rubberband while the mouse moves,
#                     erasing older and drawing newer rubberband
# insertTextLimit --> mark the other limit of the rectangle, when
#                     the button-1 is released, which erases 
#                     rubberband and makes us ready to run
#                     insertText (where the real text comes)
#
proc insertTextMark {x y} {
	global sld boxedtext
	set sld(page,update) 1
	if {$boxedtext == 0} {
		insertText $x $y
	} else {
		snap x y
		set sld(x0) $x
		set sld(y0) $y
	}
}

proc insertTextDrag {x y} {
	global sld boxedtext
	if {$boxedtext} {
		snap x y
		.c.c delete rubberband
		.c.c create rectangle $sld(x0) $sld(y0) $x $y -outline $sld(foreground) \
			-tags rubberband
	}
}

proc insertTextLimit {x y} {
	global sld boxedtext
	if {$boxedtext} {
		snap x y
		.c.c delete rubberband
		set sld(textwidth) [expr $x - $sld(x0)]
		insertText $sld(x0) $sld(y0)
	}
}

# new version of text insert procedures


proc txt_sel {x y} {
	global sldfnt sld giz fonte
	snap x y
	set w .c.c
	set fontdesc [list $sld(fontfamily) $sld(fontsize) $sld(fontstyle)]
	set showfnt [list $sld(fontfamily) \
		[expr round($sld(fontsize)*$sld(scale))] $sld(fontstyle)]
	
	set giz $sld(foreground)
	set fonte $showfnt
	txt_done
	set it [$w find overlapping [expr $x-10] [expr $y-10] \
		[expr $x+10] [expr $y+10]]
	if {[$w type $it] == "text"} {
		$w addtag "edittext" withtag $it 
		set fontdesc [$w itemcget $it -font]
	} else {
		$w config -insertbackground $giz
		$w create text $x [expr $y-[font metrics $fonte -linespace]] \
			-fill $giz -font $fonte -anchor nw \
			-justify $sld(txtalign) -tags {slide edittext}
	}
	focus $w
	$w focus "edittext"
	$w icursor "edittext" @$x,$y
	set sldfnt([$w find withtag edittext]) $fontdesc
}

proc txt_move_insert {dir} {
	set w .c.c
	set i [$w index edittext insert]
	if {$dir == "left"} {
		if {$i} {
			$w icursor edittext [expr $i-1]
		}
	} else {
		$w icursor edittext [expr $i+1]
	}
}

proc txt_add {str} {
	.c.c insert "edittext" insert $str
}

proc txt_done {} {
	global sld
	set sld(page,update) 1
	set w .c.c
	set mesg [$w itemcget "edittext" -text]
	if {[string length [string trim $mesg]] == 0} {
		$w delete "edittext"
	}
	set sld(obj) [$w find withtag edittext]
	$w dtag "edittext"
	$w focus ""
	focus .
}

proc txt_bks {} { 
	set w .c.c 
	set pos [expr [$w index "edittext" insert]-1]
	if {$pos >= 0} { 
		$w dchars "edittext" $pos
	} 
}

#-----------------------------------------------------
proc insertLineEnd {x y} {
	global sld
	snap x y
	.c.c delete rubberband
### testing sld(lstyle) for dash-patch enabled (or no line style).
### Not so bright, but very straightforward.
	if {$sld(lstyle) == " "} {
		set sld(obj) [.c.c create line $sld(x0) $sld(y0) \
			$x $y -fill $sld(foreground) \
			-width $sld(lwidth) -tags slide]
	} else {
		set sld(obj) [.c.c create line $sld(x0) $sld(y0) \
			$x $y -fill $sld(foreground) -dash $sld(lstyle)\
			-width $sld(lwidth) -tags slide]
	}
}

proc insertLineMove {x y} {
	global sld
	snap x y
	.c.c delete rubberband
	.c.c create line $sld(x0) $sld(y0) $x $y -fill $sld(foreground) \
		-width $sld(lwidth) -tags rubberband
}

proc insertLine {x y} {
	global sld
	set sld(page,update) 1
	snap x y
	.c.c config -cursor lr_angle
	set sld(x0) $x
	set sld(y0) $y
}

proc insertEllipseEnd {x y} {
	global sld fillbox
	snap x y
	.c.c delete rubberband
	if {$fillbox} {
		set sld(obj) [.c.c create oval $sld(x0) $sld(y0) \
			$x $y -outline $sld(foreground) \
			-fill $sld(fillcolor) -width $sld(lwidth) -tags slide]
	} {
		set sld(obj) [.c.c create oval $sld(x0) $sld(y0) \
			$x $y -outline $sld(foreground) \
			-width $sld(lwidth) -tags slide]
	}
}

proc insertEllipseMove {x y} {
	global sld
	snap x y
	.c.c delete rubberband
	.c.c create oval  $sld(x0) $sld(y0) $x $y -outline $sld(foreground) \
		-width $sld(lwidth) -tags rubberband
}

proc insertBoxEnd {x y} {
	global sld fillbox
	snap x y
	.c.c delete rubberband
	if {$fillbox} {
		set sld(obj) [.c.c create rectangle $sld(x0) $sld(y0) \
			$x $y -outline $sld(foreground) \
			-fill $sld(fillcolor) -width $sld(lwidth) -tags slide]
	} {
		set sld(obj) [.c.c create rectangle $sld(x0) $sld(y0) \
			$x $y -outline $sld(foreground) \
			-width $sld(lwidth) -tags slide]
	}
}

proc insertBoxMove {x y} {
	global sld
	snap x y
	.c.c delete rubberband
	.c.c create rectangle $sld(x0) $sld(y0) $x $y -outline $sld(foreground) \
		-width $sld(lwidth) -tags rubberband
}

