proc hiliteItem {x y} {
	global sld cv
	incr x [expr int($sld(xviewfrac) * [$cv cget -width])]
	incr y [expr int($sld(yviewfrac) * [$cv cget -height])]
	unhiliteItem
	set it [$cv find closest $x $y 20]
	set sld(hilited) $it
	catch {
		set sld(hilited,fill) [$cv itemcget $it -fill]
		$cv itemconfig $it -fill $sld(hilitecolor)
	}	
}

proc unhiliteItem {} {
	global sld cv
	if {[info exists sld(hilited)]} {
		catch {$cv itemconfig $sld(hilited) -fill $sld(hilited,fill)}
		unset sld(hilited) 
	}
}

proc menuGotoSlide {} {
	global sld
	set m .mb.view.m.goto
	$m delete 0 end
	for {set i 1} {$i <= $sld(page,max)} {incr i} {
		if {![expr ($i-1)%4]} {
			$m add command -label $i -command "gotoSlide $i" \
				-columnbreak 1
		} else {
			$m add command -label $i -command "gotoSlide $i" 
		}
	}
}

proc setupCanvasSize { } {
    global sld cv
	$cv config -height $sld(height) -width $sld(width) \
		-scrollregion [list -1c -1c \
		[expr round($sld(dwidth)*$sld(scale))+30] \
		[expr round($sld(dheight)*$sld(scale))+30]]
}

proc resizedCanvas { } {
	global sld
	set sld(width) [.c.c cget -width]
	set sld(height) [.c.c cget -height]
}

proc setFontDialog { } {
	global sld
	set fnt [chooseFont]
	set sld(fontfamily) [lindex $fnt 0]
	set sld(fontsize) [lindex $fnt 1]
	set sld(fontstyle) [join [lrange $fnt 2 end]]
}
 
proc dashPatchCheck { } {
	global sld cv
	set error [catch {$cv create line 0 0 1 0 -dash {.} -tags tmp}]
	if {$error} {
# dash patch not found -- disable line styles
		.mb.layout.m.obj.sty delete 2 end						
		set sld(lstyle) ""
	}
	catch {$cv delete withtag tmp}
}

proc resizeCanvas { } {
	global xtopadd ytopadd
	regexp {([0-9]*)x([0-9]*)\+.*} \
		[wm geometry .] all topwidth topheight rest
	.c.c config -width [expr $topwidth - $xtopadd] \
		-height [expr $topheight - $ytopadd]
}

proc showHints { w {y {}} } {
	global hint
	if {[string length $y] > 0} {
		set t [$w type @$y]
		set ml {cascade command checkbutton radiobutton}
		if {[lsearch $ml $t] >= 0} {
			set e [$w index @$y] 
			set h [$w entrycget @$y -label]
			if ![info exists hint($w,$e)] {
				set hint($w,$e) $h
			}
			.st config -text $hint($w,$e)
		}
	} {
		if ![info exists hint($w)] {
			set hint($w) $w
		}
		.st config -text $hint($w)
	}
}

proc setCursor {c} {
	global sld
	set sld(cursor) $c
	.c.c config -cursor $c
}

## This implements a "last visited" files list
## to be easier to select previously visited files

proc fileListUpdate { {fn ""} } {
	global sld
##### if fn is given, we must put it in front of sld(files) list
	if {[string length $fn] > 0} {
		set n [lsearch $sld(files) $fn]
##### if fn already there, remove it from list and put in top
		if {$n >=0} { 
			set list1 [lrange $sld(files) 0 [expr $n-1]]
			set list2 [lrange $sld(files) [expr $n+1] end]
			set sld(files) [concat $list1 $list2]
		} 
		set sld(files) [linsert $sld(files) 0 $fn]
		if {[llength $sld(files)] > 7} {
			set sld(files) [lrange $sld(files) 0 6]
		}
	}
##### see if anything to be done refreshing the menu
##### first, we delete old menu bottom lines
	if {[.mb.file.m index end] > 10} {
		.mb.file.m delete 10 end
	}
##### now we put new menu entries for each file
	set i 1
	foreach fn $sld(files) { 
		set lab $fn
##### truncate filename to 20 chars and add "..." at the beginning
		if {[string length $fn] > 20} {
			set lab "...[string range $fn \
				[expr [string length $fn]-18] end]"
		}
		.mb.file.m add command -label "$i $lab" -underline 0 \
			-command "fileLoad $fn" 
		incr i
	}
}


proc saveConfig {} {
	global sld
	set fd [open ~/.slidedrawrc w]
	foreach i [array names sld] {
		puts $fd "set sld($i) \{$sld($i)\}"
	}
	close $fd
### give a chance to the user save his file
	if {($sld(update)||$sld(page,update)) && \
		([tk_dialog .svquit "Leaving Slidedraw" \
			"Do you want to save current slide file before exiting?" \
			questhead 0 Save Quit] == 0)} {
				fileSave
	} 
### clean slidedraw's tmp directory
	cleanSlideDir
}

#
# snap -- round x and y to nearest snapped values
# (beware this changes the passed parameters, NOT a local copy)
# 
proc snap { xl yl } {
	global sld
	upvar $xl x
	upvar $yl y
# here's a better way of doing all messy calculations (simple indeed!)
	set x [.c.c canvasx $x $sld(snap)]
	set y [.c.c canvasy $y $sld(snap)]
	set x [expr ($x < 0) ? 0 : $x]
	set y [expr ($y < 0) ? 0 : $y]
}

####DavidL: page

# Here we set paper size and orientation.
proc page {} {
	global sld dina4 units
	# new window
	toplevel .pag
	wm title .pag "Page Setup"
	wm transient .pag .
	wm geometry .pag 255x180

	# new frame, top options
	frame .pag.f
	# orientation of page: portrait or landscape
	# this is just for current slide. Default should be changed in a separate
	# "Configure..." popup
	radiobutton .pag.f.port -text "Portrait" -variable pageorient\
		-value "port"
	radiobutton .pag.f.lands -text "Landscape" -variable pageorient\
		-value "lands"
	# Landscape is default
	.pag.f.lands invoke
	pack .pag.f.port .pag.f.lands -fill x -expand 1
	pack .pag.f.port -side left
	pack .pag.f.lands -side right

	# now units and size
	frame .pag.f11
	radiobutton .pag.f11.unit-mm -text Milimeters -variable units -value m
	radiobutton .pag.f11.unit-in -text Inches -variable units -value i
	# milimeters is default
	.pag.f11.unit-mm invoke

	pack .pag.f11.unit-mm .pag.f11.unit-in -fill x -expand 1
	pack .pag.f11.unit-mm -side left
	pack .pag.f11.unit-in -side right
	
	# here starts page dimensions
	frame .pag.f121	
	entry .pag.f121.h -width 6
	label .pag.f121.ht -text Height 
	pack .pag.f121.ht -side left
	pack .pag.f121.h -side right

	frame .pag.f122
	entry .pag.f122.w -width 6
	label .pag.f122.wt -text Width
	pack .pag.f122.wt -side left
	pack .pag.f122.w -side right

	# some default sizes
	# ISO/DIN-A4, mainly used in Europe. (ISO Standard)
	frame .pag.f13
	# checkbutton as there's only an option at the moment
	checkbutton .pag.f13.a4 -text "ISO/DIN-A4" -variable dina4 
	# I don't remember USLetter size right now.
	# Support for other popular sizes (like ISO/DIN-A5, half an A4, US
	# Legal. &c) can be easily added, but I have to look for the american
	# papers first. Using radiobuttons may pose a problem, as if we check
	# one of them then we cannot uncheck it...

	#### radiobutton .pag.f12.letter -text "US Letter" \
	####	-variable pagedefault -value letter -state disabled
	
	pack .pag.f13.a4
	# we leave US Letter out for now
	#### pack .pag.f1.letter -side bottom
	
	# window buttons
	frame .pag.f2
	button .pag.f2.accept -text Apply -command {
		if {$dina4} then {
			# we should disable these two if a standard page size
			# is set, but I can't find how to do that on runtime
			#### pack .pag.f121.h 
			#### pack .pag.f122.w -state disabled
			set tmpheight [winfo pixels .c.c 297m]
			set tmpwidth [winfo pixels .c.c 210m]
		} else {
			# should we change default values sld(height) and sld(width)? I
			# don't think so, those should be changed in a separate
			# "Configure..." popup...
            set tmpwidth [winfo pixels .c.c \
                "[string trim [.pag.f122.w get]]$units"]
			set tmpheight [winfo pixels .c.c \
                "[string trim [.pag.f121.h get]]$units"]
		}
        if {[string compare $pageorient "lands"] == 0} {
            set sld(dheight) $tmpwidth
            set sld(dwidth) $tmpheight
        } else {
            set sld(dwidth) $tmpwidth
            set sld(dheight) $tmpheight
        }
		setupCanvasSize
        makeRulers

		# if unchecked, you have to dismiss the popup, but can check if the
		# size is what you want...
		# destroy .pag
	}
		
	button .pag.f2.cancel -text Cancel -command {destroy .pag}
	pack .pag.f2 .pag.f2.accept .pag.f2.cancel -side bottom -pady 5 -padx 15
	pack .pag.f2.accept -side left -expand 1
	pack .pag.f2.cancel -side right -expand 1 
		
	pack .pag.f11 .pag.f .pag.f121 .pag.f122 .pag.f13 .pag.f2 -pady 3
	centerWindow .pag
	raise .pag
}

#
# centerWindow -- make some calculations to put the window
# at the center of the main toplevel. 
#
proc centerWindow { w } {
	update idletasks
	set x [expr [winfo rootx .]+[winfo width .]/2-[winfo width $w]/2]
	set y [expr [winfo rooty .]+[winfo height .]/2-[winfo height $w]/2]
	wm geometry $w +$x+$y
}
	
